@@ -1,34 +1,38 @@
+#![cfg_attr(not(feature = "std"), no_std)]
 #![feature(core_intrinsics)]
 #![feature(const_fn)]
 #![feature(const_generics)]
 #![feature(maybe_uninit_ref)]
 #![feature(maybe_uninit_extra)]
+#![feature(exact_size_is_empty)]
 
 use crate::utils::*;
-use std::cmp::{Ord, PartialEq};
-use std::iter::FromIterator;
-use std::marker::PhantomData;
-use std::mem::MaybeUninit;
-use std::ops::{Bound::Excluded, Bound::Included, Bound::Unbounded, Index, IndexMut, RangeBounds};
-use std::ptr;
+use core::cmp::{Ord, PartialEq};
+use core::iter::FromIterator;
+use core::marker::PhantomData;
+use core::mem::MaybeUninit;
+use core::ops::{Bound::Excluded, Bound::Included, Bound::Unbounded, Index, IndexMut, RangeBounds};
+use core::ptr;
+#[doc(hidden)]
+pub mod macro_constructor;
+mod macros;
 mod utils;
 
-///A [Vec](std::vec::Vec)-like struct (directly API-compatible where it can be
-///at least as far as function signatures go) implemented with
-///const generics around a static array of fixed `N` capacity.
+///A [Vec](std::vec::Vec)-like struct (mostly directly API-compatible where it can be)
+///implemented with const generics around a static array of fixed `N` capacity.
 pub struct StaticVec<T, const N: usize> {
   data: [MaybeUninit<T>; N],
   length: usize,
 }
 
-///Similar to std's [Iter](std::slice::IterMut), but specifically implemented with StaticVecs in mind.
+///Similar to [Iter](core::slice::IterMut), but specifically implemented with StaticVecs in mind.
 pub struct StaticVecIterConst<'a, T: 'a> {
   start: *const T,
   end: *const T,
   marker: PhantomData<&'a T>,
 }
 
-///Similar to std's [IterMut](std::slice::IterMut), but specifically implemented with StaticVecs in mind.
+///Similar to [IterMut](core::slice::IterMut), but specifically implemented with StaticVecs in mind.
 pub struct StaticVecIterMut<'a, T: 'a> {
   start: *mut T,
   end: *mut T,
@@ -51,7 +55,7 @@ impl<T, const N: usize> StaticVec<T, {N}> {
   ///Returns a new StaticVec instance filled with the contents, if any, of a slice.
   ///If the slice has a length greater than the StaticVec's capacity,
   ///any contents after that point are ignored.
-  ///Locally requires that `T` implements [Copy](std::marker::Copy) to avoid soundness issues.
+  ///Locally requires that `T` implements [Copy](core::marker::Copy) to avoid soundness issues.
   #[inline]
   pub fn new_from_slice(values: &[T]) -> Self
   where T: Copy {
@@ -70,7 +74,7 @@ impl<T, const N: usize> StaticVec<T, {N}> {
 
   ///Returns a new StaticVec instance filled with the return value of
   ///an initializer function.
-  ///`len()` will return the same as `capacity()` for the newly created StaticVec.
+  ///The length field of the newly created StaticVec will be equal to its capacity.
   #[inline]
   pub fn filled_with(initializer: fn() -> T) -> Self {
     unsafe {
@@ -224,7 +228,7 @@ impl<T, const N: usize> StaticVec<T, {N}> {
     }
   }
 
-  ///Asserts that `T`he current length of the StaticVec is less than `N` and that
+  ///Asserts that the current length of the StaticVec is less than `N` and that
   ///`index` is less than the length, and if so inserts `value` at that position.
   ///Any values that exist in later positions are shifted to the right.
   #[inline]
@@ -251,7 +255,8 @@ impl<T, const N: usize> StaticVec<T, {N}> {
   }
 
   ///Performs an stable in-place sort of the StaticVec's inhabited area.
-  ///Locally requires that `T` implements [Ord](std::cmp::Ord) to make the sorting possible.
+  ///Locally requires that `T` implements [Ord](core::cmp::Ord) to make the sorting possible.
+  #[cfg(feature = "std")]
   #[inline(always)]
   pub fn sort(&mut self)
   where T: Ord {
@@ -259,7 +264,7 @@ impl<T, const N: usize> StaticVec<T, {N}> {
   }
 
   ///Performs an unstable in-place sort of the StaticVec's inhabited area.
-  ///Locally requires that `T` implements [Ord](std::cmp::Ord) to make the sorting possible.
+  ///Locally requires that `T` implements [Ord](core::cmp::Ord) to make the sorting possible.
   #[inline(always)]
   pub fn sort_unstable(&mut self)
   where T: Ord {
@@ -274,8 +279,9 @@ impl<T, const N: usize> StaticVec<T, {N}> {
 
   ///Returns a separate, stable-sorted StaticVec of the contents of the
   ///StaticVec's inhabited area without modifying the original data.
-  ///Locally requires that `T` implements [Copy](std::marker::Copy) to avoid soundness issues,
-  ///and [Ord](std::cmp::Ord) to make the sorting possible.
+  ///Locally requires that `T` implements [Copy](core::marker::Copy) to avoid soundness issues,
+  ///and [Ord](core::cmp::Ord) to make the sorting possible.
+  #[cfg(feature = "std")]
   #[inline]
   pub fn sorted(&mut self) -> Self
   where T: Copy + Ord {
@@ -292,8 +298,8 @@ impl<T, const N: usize> StaticVec<T, {N}> {
 
   ///Returns a separate, unstable-sorted StaticVec of the contents of the
   ///StaticVec's inhabited area without modifying the original data.
-  ///Locally requires that `T` implements [Copy](std::marker::Copy) to avoid soundness issues,
-  ///and [Ord](std::cmp::Ord) to make the sorting possible.
+  ///Locally requires that `T` implements [Copy](core::marker::Copy) to avoid soundness issues,
+  ///and [Ord](core::cmp::Ord) to make the sorting possible.
   #[inline]
   pub fn sorted_unstable(&mut self) -> Self
   where T: Copy + Ord {
@@ -310,8 +316,8 @@ impl<T, const N: usize> StaticVec<T, {N}> {
 
   ///Returns a separate, reversed StaticVec of the contents of the StaticVec's
   ///inhabited area without modifying the original data.
-  ///Locally requires that `T` implements [Copy](std::marker::Copy) to avoid soundness issues.
-  #[inline(always)]
+  ///Locally requires that `T` implements [Copy](core::marker::Copy) to avoid soundness issues.
+  #[inline]
   pub fn reversed(&mut self) -> Self
   where T: Copy {
     let mut res = Self::new();
@@ -329,7 +335,7 @@ impl<T, const N: usize> StaticVec<T, {N}> {
   ///Copies and appends all elements, if any, in a slice to the StaticVec.
   ///Unlike the implementation of this function for [Vec](std::vec::Vec), no iterator is used,
   ///just a single pointer-copy call.
-  ///Locally requires that `T` implements [Copy](std::marker::Copy) to avoid soundness issues.
+  ///Locally requires that `T` implements [Copy](core::marker::Copy) to avoid soundness issues.
   #[inline]
   pub fn extend_from_slice(&mut self, other: &[T])
   where T: Copy {
@@ -379,41 +385,25 @@ impl<T, const N: usize> StaticVec<T, {N}> {
   }
 
   ///Returns a `StaticVecIterConst` over the StaticVec's inhabited area.
-  #[inline]
+  #[inline(always)]
   pub fn iter<'a>(&'a self) -> StaticVecIterConst<'a, T> {
     unsafe {
-      if self.length > 0 {
-        StaticVecIterConst::<'a, T> {
-          start: self.as_ptr(),
-          end: self.as_ptr().add(self.length),
-          marker: PhantomData,
-        }
-      } else {
-        StaticVecIterConst::<'a, T> {
-          start: self.as_ptr(),
-          end: self.as_ptr(),
-          marker: PhantomData,
-        }
+      StaticVecIterConst::<'a, T> {
+        start: self.as_ptr(),
+        end: self.as_ptr().add(self.length),
+        marker: PhantomData,
       }
     }
   }
 
   ///Returns a `StaticVecIterMut` over the StaticVec's inhabited area.
-  #[inline]
+  #[inline(always)]
   pub fn iter_mut<'a>(&'a mut self) -> StaticVecIterMut<'a, T> {
     unsafe {
-      if self.length > 0 {
-        StaticVecIterMut::<'a, T> {
-          start: self.as_mut_ptr(),
-          end: self.as_mut_ptr().add(self.length),
-          marker: PhantomData,
-        }
-      } else {
-        StaticVecIterMut::<'a, T> {
-          start: self.as_mut_ptr(),
-          end: self.as_mut_ptr(),
-          marker: PhantomData,
-        }
+      StaticVecIterMut::<'a, T> {
+        start: self.as_mut_ptr(),
+        end: self.as_mut_ptr().add(self.length),
+        marker: PhantomData,
       }
     }
   }
@@ -472,7 +462,7 @@ impl<T, const N: usize> FromIterator<T> for StaticVec<T, {N}> {
   ///Creates a new StaticVec instance from the elements, if any, of `iter`.
   ///If it has a size greater than the StaticVec's capacity, any items after
   ///that point are ignored.
-  #[inline]
+  #[inline(always)]
   fn from_iter<I: IntoIterator<Item = T>>(iter: I) -> Self {
     let mut res = Self::new();
     for value in iter {
@@ -490,20 +480,19 @@ impl<T, const N: usize> FromIterator<T> for StaticVec<T, {N}> {
 
 impl<'a, T: 'a> Iterator for StaticVecIterConst<'a, T> {
   type Item = &'a T;
-  ///Returns `Some(&*self.start)` if `start` is less than `end`,
-  ///and `None` if it's not.
   #[inline(always)]
   fn next(&mut self) -> Option<Self::Item> {
     if self.start < self.end {
       unsafe {
         let res = Some(&*self.start);
-        self.start = self.start.add(1);
+        self.start = self.start.offset(1);
         res
       }
     } else {
       None
     }
   }
+
   #[inline(always)]
   fn size_hint(&self) -> (usize, Option<usize>) {
     let len = distance_between(self.end, self.start);
@@ -512,14 +501,12 @@ impl<'a, T: 'a> Iterator for StaticVecIterConst<'a, T> {
 }
 
 impl<'a, T: 'a> DoubleEndedIterator for StaticVecIterConst<'a, T> {
-  ///Returns `Some(&*self.end)` if `end` is greater than `start`,
-  ///and `None` if it's not.
   #[inline(always)]
   fn next_back(&mut self) -> Option<Self::Item> {
     if self.end > self.start {
       unsafe {
+        self.end = self.end.offset(-1);
         let res = Some(&*self.end);
-        self.end = self.end.sub(1);
         res
       }
     } else {
@@ -528,24 +515,33 @@ impl<'a, T: 'a> DoubleEndedIterator for StaticVecIterConst<'a, T> {
   }
 }
 
-impl<'a, T: 'a> ExactSizeIterator for StaticVecIterConst<'a, T> {}
+impl<'a, T: 'a> ExactSizeIterator for StaticVecIterConst<'a, T> {
+  #[inline(always)]
+  fn len(&self) -> usize {
+    distance_between(self.end, self.start)
+  }
+
+  #[inline(always)]
+  fn is_empty(&self) -> bool {
+    distance_between(self.end, self.start) == 0
+  }
+}
 
 impl<'a, T: 'a> Iterator for StaticVecIterMut<'a, T> {
   type Item = &'a mut T;
-  ///Returns `Some(&mut *self.start)` if `start` is less than `end`,
-  ///and `None` if it's not.
   #[inline(always)]
   fn next(&mut self) -> Option<Self::Item> {
     if self.start < self.end {
       unsafe {
         let res = Some(&mut *self.start);
-        self.start = self.start.add(1);
+        self.start = self.start.offset(1);
         res
       }
     } else {
       None
     }
   }
+
   #[inline(always)]
   fn size_hint(&self) -> (usize, Option<usize>) {
     let len = distance_between(self.end, self.start);
@@ -554,14 +550,12 @@ impl<'a, T: 'a> Iterator for StaticVecIterMut<'a, T> {
 }
 
 impl<'a, T: 'a> DoubleEndedIterator for StaticVecIterMut<'a, T> {
-  ///Returns `Some(&mut *self.end)` if `end` is greater than `start`,
-  ///and `None` if it's not.
   #[inline(always)]
   fn next_back(&mut self) -> Option<Self::Item> {
     if self.end > self.start {
       unsafe {
+        self.end = self.end.offset(-1);
         let res = Some(&mut *self.end);
-        self.end = self.end.sub(1);
         res
       }
     } else {
@@ -570,4 +564,14 @@ impl<'a, T: 'a> DoubleEndedIterator for StaticVecIterMut<'a, T> {
   }
 }
 
-impl<'a, T: 'a> ExactSizeIterator for StaticVecIterMut<'a, T> {}
+impl<'a, T: 'a> ExactSizeIterator for StaticVecIterMut<'a, T> {
+  #[inline(always)]
+  fn len(&self) -> usize {
+    distance_between(self.end, self.start)
+  }
+
+  #[inline(always)]
+  fn is_empty(&self) -> bool {
+    distance_between(self.end, self.start) == 0
+  }
+}
